@@ -6,8 +6,11 @@ knitr::opts_chunk$set(
 )
 
 ## ----sim_ctns------------------------------------------------------------
+library(ggplot2)
+library(dplyr)
+library(partykit)
 library(StratifiedMedicine)
-dat_ctns = generate_subgrp_data(family="gaussian")
+dat_ctns = generate_subgrp_data(family="gaussian", seed=65532)
 Y = dat_ctns$Y
 X = dat_ctns$X # 50 covariates, 46 are noise variables, X1 and X2 are truly predictive
 A = dat_ctns$A # binary treatment, 1:1 randomized 
@@ -19,17 +22,71 @@ dim(X)
 # PRISM Default: filter_glmnet, ple_ranger, submod_lmtree, param_ple #
 res0 = PRISM(Y=Y, A=A, X=X)
 ## This is the same as running ##
-res1 = PRISM(Y=Y, A=A, X=X, family="gaussian", filter="filter_glmnet", 
-             ple = "ple_ranger", submod = "submod_lmtree", param="param_ple")
-## Plot the distribution of PLEs ###
-hist(res0$mu_train$PLE, main="Estimated Distribution of PLEs",
-     xlab = "Estimated PLEs: E(Y|X=x, A=1)-E(Y|X=x,A=0)")
-## Plot of the subgroup model (lmtree) ##
-plot(res0$Sub.mod)
+# res1 = PRISM(Y=Y, A=A, X=X, family="gaussian", filter="filter_glmnet", 
+#              ple = "ple_ranger", submod = "submod_lmtree", param="param_ple")
+
+## ----default_ctns_filter-------------------------------------------------
+# elastic net model: loss by lambda #
+plot(res0$filter.mod)
+## Variables that remain after filtering ##
+res0$filter.vars
+# All predictive variables (X1,X2) and prognostic variables (X3,X5, X7) remains.
+
+## ----default_ctns_ple----------------------------------------------------
+prob.PLE = mean(I(res0$mu_train$PLE>0))
+# Density Plot #
+plot(res0, type="PLE:density")+geom_vline(xintercept = 0) +
+     geom_text(x=1.5, y=0.4, label=paste("Prob(PLE>0)=", prob.PLE, sep=""))
+# Waterfall Plot #
+plot(res0, type="PLE:waterfall")+geom_vline(xintercept = 0) + 
+  geom_text(x=200, y=1, label=paste("Prob(PLE>0)=", prob.PLE, sep=""))
+
+## ----default_ctns_submod-------------------------------------------------
+plot(res0$submod.fit$mod, terminal_panel = NULL)
+table(res0$out.train$Subgrps)
+table(res0$out.test$Subgrps)
+
+## ----default_ctns2-------------------------------------------------------
 ## Overall/subgroup specific parameter estimates/inference
 res0$param.dat
 ## Forest plot: Overall/subgroup specific parameter estimates (CIs)
-plot(res0)
+plot(res0, type="forest")
+
+## ----modify_submod_plot--------------------------------------------------
+param.dat = res0$param.dat[res0$param.dat$Subgrps>0,]
+param.dat$est = with(param.dat, sprintf("%.2f",round(est,2)))
+param.dat$CI = with(param.dat, paste("[", sprintf("%.2f",round(param.dat$LCL,2)),",",
+                                     sprintf("%.2f",round(param.dat$UCL,2)),"]",sep=""))
+mu_1 = aggregate(res0$mu_train$mu1~res0$out.train$Subgrps, FUN="mean")
+colnames(mu_1) = c("Subgrps", "est.A1")
+mu_0 = aggregate(res0$mu_train$mu0~res0$out.train$Subgrps, FUN="mean")
+colnames(mu_0) = c("Subgrps", "est.A0")
+param.dat = left_join(param.dat, mu_1, by="Subgrps")
+param.dat = left_join(param.dat, mu_0, by="Subgrps")
+param.dat$est.A1 = with(param.dat, sprintf("%.2f",round(est.A1,2)))
+param.dat$est.A0 = with(param.dat, sprintf("%.2f",round(est.A0,2)))
+smod = res0$submod.fit$mod
+smod_node <- as.list(smod$node)
+for(i in 1:nrow(param.dat)){
+   smod_node[[param.dat[i,1]]]$info$est <- param.dat$est[i]
+   smod_node[[param.dat[i,1]]]$info$CI <-  param.dat$CI[i]
+   smod_node[[param.dat[i,1]]]$info$est.A1 <- param.dat$est.A1[i]
+   smod_node[[param.dat[i,1]]]$info$est.A0 <- param.dat$est.A0[i]
+}
+smod$node <- as.partynode(smod_node)
+plot(smod, terminal_panel = node_terminal, tp_args = list(
+  FUN = function(node) c( paste("n =", node$nobs),
+                          paste("E(Y|A=0):", node$est.A0),
+                          paste("E(Y|A=1):", node$est.A1),
+                          paste("Diff:",node$est),
+                          node$CI) ) )
+
+
+## ----heat_maps-----------------------------------------------------------
+grid.data = expand.grid(X1 = seq(min(X$X1), max(X$X1), by=0.30),
+                    X2 = seq(min(X$X2), max(X$X2), by=0.30))
+plot(res0, type="heatmap", grid.data = grid.data)
+
 
 ## ----default_hyper-------------------------------------------------------
 # PRISM Default: filter_glmnet, ple_ranger, submod_lmtree, param_ple #
@@ -37,11 +94,12 @@ plot(res0)
 res_new_hyper = PRISM(Y=Y, A=A, X=X, filter.hyper = list(lambda="lambda.1se"),
                       ple.hyper = list(min.node.pct=0.05), 
                       submod.hyper = list(minsize=200))
-plot(res_new_hyper$Sub.mod) # Plot subgroup model results
-plot(res_new_hyper)
+plot(res_new_hyper$submod.fit$mod) # Plot subgroup model results
+plot(res_new_hyper) # Forest plot 
 
 ## ----default_boot--------------------------------------------------------
 library(ggplot2)
+library(dplyr)
 res_boot = PRISM(Y=Y, A=A, X=X, resample = "Bootstrap", R=50, verbose=FALSE)
 # # Plot of distributions and P(est>0) #
 plot(res_boot, type="resample")+geom_vline(xintercept = 0)
@@ -60,13 +118,14 @@ X = surv.dat[,!(colnames(surv.dat) %in% c("time", "cens")) ]
 A = rbinom( n = dim(X)[1], size=1, prob=0.5  )
 
 # Default: filter_glmnet ==> submod_weibull (MOB with Weibull) ==> param_cox (Cox regression)
-res_weibull1 = PRISM(Y=Y, A=A, X=X, ple=NULL)
-plot(res_weibull1$Sub.mod)
+res_weibull1 = PRISM(Y=Y, A=A, X=X)
+plot(res_weibull1, type="PLE:waterfall")
+plot(res_weibull1$submod.fit$mod)
 plot(res_weibull1)+ylab("HR [95% CI]")
 
 # PRISM: filter_glmnet ==> submod_ctree ==> param_cox (Cox regression) #
 res_ctree1 = PRISM(Y=Y, A=A, X=X, ple=NULL, submod = "submod_ctree")
-plot(res_ctree1$Sub.mod)
+plot(res_ctree1$submod.fit$mod)
 plot(res_ctree1)+ylab("HR [95% CI]")
 
 
@@ -92,7 +151,7 @@ filter_lasso = function(Y, A, X, lambda="lambda.min", family="gaussian", ...){
   VI <- coef(mod, s = lambda)[,1]
   VI = VI[-1]
   filter.vars = names(VI[VI!=0])
-  return( list(mod=mod, filter.vars=filter.vars) )
+  return( list(filter.vars=filter.vars) )
 }
 
 ## ----user_ple------------------------------------------------------------
@@ -114,6 +173,8 @@ ple_ranger_mtry = function(Y, A, X, Xtest, mtry=5, ...){
     mu_test = data.frame( mu1 =  predict(mod1, data = Xtest)$predictions,
                             mu0 = predict(mod0, data = Xtest)$predictions)
     mu_test$PLE = with(mu_test, mu1 - mu0 )
+    res = list(mods=mods, mu_train=mu_train, mu_test=mu_test)
+    class(res) = "ple_ranger_mtry"
     return( list(mods=mods, mu_train=mu_train, mu_test=mu_test))
 }
 
@@ -180,7 +241,7 @@ res_user1 = PRISM(Y=Y, A=A, X=X, family="gaussian", filter="filter_lasso",
 ## variables that remain after filtering ##
 res_user1$filter.vars
 ## Subgroup model: lmtree searching for predictive only ##
-plot(res_user1$Sub.mod)
+plot(res_user1$submod.fit$mod)
 ## Parameter estimates/inference
 res_user1$param.dat
 ## Forest Plot (95% CI) ##
